#!/usr/bin/python3
# -*- coding: utf-8 -*-
from __future__ import absolute_import

import os
import random
import string

from django.conf import settings
from django.template.defaultfilters import slugify
from django.utils.module_loading import import_string

IMAGE_EXTENSIONS = {'jpg', 'jpeg', 'png', 'gif'}


# Allow for a custom storage backend defined in settings.
# https://docs.djangoproject.com/en/3.0/ref/files/storage/
def get_storage_class():

    return import_string(getattr(settings, 'WANGEDITOR_STORAGE_BACKEND', 'django.core.files.storage.DefaultStorage'))()


storage = get_storage_class()


def slugify_filename(filename):
    """ Slugify filename """
    name, ext = os.path.splitext(filename)
    slugified = get_slugified_name(name)
    return slugified + ext


def get_slugified_name(filename):
    """
    slugify: 转换为ASCII。将空格转换为连字符。删除字符
    不是字母数字、下划线或连字符。转换为小写的。
    还可以删除开头和结尾的空白。
    """
    slugified = slugify(filename)
    return slugified or get_random_string()


def get_random_string():
    return ''.join(random.sample(string.ascii_lowercase * 6, 6))


def get_media_url(path):
    """
    Determine system file's media URL.
    """
    return storage.url(path)


def is_valid_image_extension(file_name):
    """验证是否为图片"""
    return file_name.split('.')[-1] in IMAGE_EXTENSIONS if file_name else False



#!/usr/bin/python3
# -*- coding: utf-8 -*-
from __future__ import absolute_import

import os
from datetime import datetime

from django.views import generic

from wangeditor.backends import registry
from wangeditor.utils import storage, slugify_filename, get_media_url
from django.conf import settings
from django.http import JsonResponse, HttpResponse
from django.views.decorators.csrf import csrf_exempt


def get_upload_filename(upload_name):

    # Generate date based path to put uploaded file.
    # If CKEDITOR_RESTRICT_BY_DATE is True upload file to date specific path.
    if getattr(settings, 'WANGEDITOR_RESTRICT_BY_DATE', True):
        date_path = datetime.now().strftime('%Y/%m/%d')
    else:
        date_path = ''
    WANGEDITOR_UPLOAD_PATH = getattr(settings, 'WANGEDITOR_UPLOAD_PATH', 'uploads/')
    # Complete upload path (upload_path + date_path).
    upload_path = os.path.join(WANGEDITOR_UPLOAD_PATH, date_path)
    if getattr(settings, 'WANGEDITOR_UPLOAD_SLUGIFY_FILENAME', True):
        upload_name = slugify_filename(upload_name)

    # 返回基于name参数的文件名称，它在目标储存系统中可用于写入新的内容。
    # 如果提供了max_length，文件名称长度不会超过它。如果不能找到可用的、唯一的文件名称，会抛出SuspiciousFileOperation 异常。
    # 如果name命名的文件已存在，一个下划线加上随机7个数字或字母的字符串会添加到文件名称的末尾，扩展名之前。
    return storage.get_available_name(
        os.path.join(upload_path, upload_name)
    )


class ImageUploadView(generic.View):
    http_method_names = ['post']

    def post(self, request, **kwargs):
        """
        Uploads a file and send back its URL to WANGEditor.
        """
        uploaded_files = request.FILES

        backend = registry.get_backend()
        url_list = []
        err_no = 0
        msg = 'success'
        for file_name, uploaded_file in uploaded_files.items():
            file_wrapper = backend(storage, uploaded_file)
            if not file_wrapper.is_image:
                err_no = 1
                msg = '%s is invalid file type' % file_name
                break
            filepath = get_upload_filename(file_name)
            saved_path = file_wrapper.save_as(filepath)
            url = get_media_url(saved_path)
            url_list.append(url)
        ret_data = {
            "errno": err_no,
            "data": url_list,
            'msg': msg
        }
        return JsonResponse(ret_data)


upload = csrf_exempt(ImageUploadView.as_view())


#!/usr/bin/python3
# -*- coding: utf-8 -*-

from __future__ import absolute_import

from django import forms
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.core.serializers.json import DjangoJSONEncoder
from django.utils.encoding import force_text
from django.utils.functional import Promise
from django.utils.safestring import mark_safe

from js_asset import JS

from .configs import WANG_DEFAULT_CONFIG

try:
    # Django >=1.11
    from django.forms.widgets import get_default_renderer
except ImportError:
    # Django <1.11
    from django.template.loader import render_to_string

    def get_default_renderer():
        class DummyDjangoRenderer(object):
            @staticmethod
            def render(*args, **kwargs):
                return render_to_string(*args, **kwargs)

        return DummyDjangoRenderer

try:
    # Django >=1.7
    from django.forms.utils import flatatt
except ImportError:
    # Django <1.7
    from django.forms.util import flatatt


class LazyEncoder(DjangoJSONEncoder):

    def default(self, obj):
        if isinstance(obj, Promise):
            return force_text(obj)
        return super(LazyEncoder, self).default(obj)


json_encode = LazyEncoder().encode


class WangEditorWidget(forms.Textarea):

    class Media:
        # js
        js = (
            JS('wangeditor/wangeditor-init.js', {
                'id': 'wangeditor-init-script',
            }),
            'wangeditor/wangEditor.min.js',
        )

    def __init__(self, config_name='default',  *args, **kwargs):
        super(WangEditorWidget, self).__init__(*args, **kwargs)
        # Setup config from defaults.
        self.config = WANG_DEFAULT_CONFIG.copy()

        # Try to get valid config from settings.
        configs = getattr(settings, 'WANGEDITOR_CONFIGS', None)
        if configs:
            if isinstance(configs, dict):
                # Make sure the config_name exists.
                if config_name in configs:
                    config = configs[config_name]
                    # Make sure the configuration is a dictionary.
                    if not isinstance(config, dict):
                        raise ImproperlyConfigured('WANGEDITOR_CONFIGS["%s"] \
                                setting must be a dictionary type.' %
                                                   config_name)
                    # Override defaults with settings config.
                    self.config.update(config)
                else:
                    raise ImproperlyConfigured("No configuration named '%s' \
                            found in your WANGEDITOR_CONFIGS setting." %
                                               config_name)
            else:
                raise ImproperlyConfigured('WANGEDITOR_CONFIGS setting must be a\
                        dictionary type.')

    def render(self, name, value, attrs=None, renderer=None):
        if renderer is None:
            renderer = get_default_renderer()
        if value is None:
            value = ''
        final_attrs = self.build_attrs(self.attrs, attrs, name=name)
        return mark_safe(renderer.render('wangeditor/widget.html', {
            'final_attrs': flatatt(final_attrs), # flatatt  设置html 的属性, 形式key=value
            'value': force_text(value),
            'config': json_encode(self.config),
            'id': final_attrs['id'],
        }))

    def build_attrs(self, base_attrs, extra_attrs=None, **kwargs):
        """
        Helper function for building an attribute dictionary.
        This is combination of the same method from Django<=1.10 and Django1.11+
        """
        attrs = dict(base_attrs, **kwargs)
        if extra_attrs:
            attrs.update(extra_attrs)
        return attrs


from django.conf import settings

from .dummy_backend import DummyBackend


class BackendRegistry(object):
    def __init__(self):
        self._registry = {}

    def register(self, backend_id, backend):
        if backend_id in self._registry:
            raise KeyError('%s is already a registered' % backend_id)
        self._registry[backend_id] = backend

    def get_backend(self):
        backend_id = getattr(settings, "WANGEDITOR_IMAGE_BACKEND", None)
        if backend_id is None:
            return DummyBackend
        return self._registry[backend_id]


registry = BackendRegistry()





# #!/usr/bin/python3
# # -*- coding: utf-8 -*-


WANG_DEFAULT_CONFIG = {

    'menus': ['head', 'bold', 'fontSize', 'fontName', 'italic', 'underline', 'strikeThrough', 'foreColor', 'backColor',
              'link', 'list', 'justify', 'quote', 'emoticon', 'image', 'table', 'video', 'code', 'undo', 'redo'],
    'pasteFilterStyle': True,  # 是否关闭粘贴样式的过滤
    'pasteIgnoreImg': False,  # 是否忽略粘贴内容中的图片
    'colors': [
        '#000000',
        '#eeece0',
        '#1c487f',
        '#4d80bf',
        '#c24f4a',
        '#8baa4a',
        '#7b5ba1',
        '#46acc8',
        '#f9963b',
        '#ffffff'],  # 自定义配置颜色（字体颜色、背景色）可以添加更多的色号
}



from __future__ import absolute_import

from django.utils.functional import cached_property

from wangeditor import utils


class DummyBackend(object):
    def __init__(self, storage_engine, file_object):
        self.file_object = file_object
        self.storage_engine = storage_engine

    def save_as(self, filepath):
        return self.storage_engine.save(filepath, self.file_object)

    @cached_property
    def is_image(self):
        return utils.is_valid_image_extension(self.file_object.name)



#!/usr/bin/python3
# -*- coding: utf-8 -*-
# @Time    : 20-3-17 上午10:41
# @Author  : Hubery
# @File    : fields.py
# @Software: PyCharm

from __future__ import absolute_import

from django import forms
from django.db import models

from .widgets import WangEditorWidget


class WangRichTextField(models.TextField):

    # 表单字段
    def formfield(self, **kwargs):
        defaults = {
            'form_class': self._get_form_class(),
        }
        defaults.update(kwargs)
        return super(WangRichTextField, self).formfield(**defaults)

    @staticmethod
    def _get_form_class():
        return WangRichTextFormField


class WangRichTextFormField(forms.fields.CharField):

    def __init__(self,  *args, **kwargs):
        kwargs.update({'widget': WangEditorWidget()})
        super(WangRichTextFormField, self).__init__(*args, **kwargs)



import os
import sys
from setuptools import setup

if sys.argv[-1] == 'publish':
    os.system('python setup.py sdist bdist_wheel')
    os.system('python -m twine upload dist/*')
    sys.exit()

setup()



#!/usr/bin/python3
# -*- coding: utf-8 -*-
from __future__ import absolute_import

from django.conf.urls import url
from django.contrib.admin.views.decorators import staff_member_required  # 验证用户是否已登录
from . import views

urlpatterns = [
    url(r'^upload/$', staff_member_required(views.upload), name='upload')
]